/*
 * Velleman K8048 Programmer for FreeBSD and others.
 *
 * Copyright (c) 2005-2013 Darron Broad
 * All rights reserved.
 *
 * Licensed under the terms of the BSD license, see file LICENSE for details.
 *
 * See README.raspi
 */

#include "k8048.h"

/*
 * Open raspberry-pi gpio memory
 */
int
gpio_open(GPIO *gpio)
{
#if ENABLE_RPI==0
	printf("%s: fatal error: raspberry pi support is disabled\n", __func__);
	exit(EX_OSERR); /* Panic */
#endif
	/* Already open? */
	if (gpio->fd >= 0)
		return -1;

	/* Open /dev/mem */
	gpio->fd = open("/dev/mem", O_RDWR | O_SYNC);
	if (gpio->fd < 0)
		return -1;

	/* Memory map GPIO */
	gpio->map = mmap(NULL, GPIO_MAP_LEN, PROT_READ | PROT_WRITE, MAP_SHARED, gpio->fd, GPIO_BASE_ADDR);
	if (gpio->map == MAP_FAILED) {
		printf("%s: fatal error: mmap failed [%s]\n", __func__, strerror(errno));
		exit(EX_OSERR); /* Panic */
	}
	return 0;
}

int
gpio_close(GPIO *gpio)
{
	if (gpio->fd < 0)
		return -1;

	if (munmap(gpio->map, GPIO_MAP_LEN)) {
		printf("%s: warning: munmap failed\n", __func__);
	}
	gpio->map = NULL;

	if (close(gpio->fd)) {
		printf("%s: warning: close failed\n", __func__);
	}
	gpio->fd = -1;

	return 0;
}

int
gpio_read(GPIO *gpio, uint8_t gpio_reg, uint32_t *val)
{
	if (gpio->fd < 0)
		return -1;

	GPIO_ADDR reg = (GPIO_ADDR)(gpio->map) + gpio_reg;

	*val = *reg;

	return 0;
}

int
gpio_write(GPIO *gpio, uint8_t gpio_reg, uint32_t val)
{
	if (gpio->fd < 0)
		return -1;

	GPIO_ADDR reg = (GPIO_ADDR)(gpio->map) + gpio_reg;

	*reg = val;

	return 0;
}

int
gpio_pud(GPIO *gpio, uint8_t pin, uint8_t pud)
{
	if (gpio->fd < 0 || pin > 31)
		return -1;

	gpio_write(gpio, GPPUD, pud);
	usleep(10); /* ? */
	gpio_write(gpio, GPPUDCLK0, (1 << pin));
	usleep(10); /* ? */
	gpio_write(gpio, GPPUD, 0);
	gpio_write(gpio, GPPUDCLK0, 0);

	return 0;
}

int
gpio_get(GPIO *gpio, uint8_t pin, uint8_t *level)
{
	if (gpio->fd < 0 || pin > 31)
		return -1;

	uint32_t val;
	gpio_read(gpio, GPLEV0, &val);

	*level = (val & (1 << pin)) ? (HIGH) : (LOW);

	return 0;
}

int
gpio_set(GPIO *gpio, uint8_t pin, uint8_t level)
{
	if (gpio->fd < 0 || pin > 31)
		return -1;

	return gpio_write(gpio, (level) ? (GPSET0) : (GPCLR0), (1 << pin));
}

static inline uint32_t
gpio_gpfsel(uint8_t pin)
{
	return /* GPFSEL0 + */(pin / 10);
}

int
gpio_select_input(GPIO *gpio, uint8_t pin, uint8_t pud)
{
	if (gpio->fd < 0 || pin > 31)
		return -1;

	GPIO_ADDR reg = (GPIO_ADDR)(gpio->map) + gpio_gpfsel(pin);

	uint32_t val = ~(7 << ((pin % 10) * 3));
	*reg &= val; /* 000 = Input */

	return gpio_pud(gpio, pin, pud);
}

int
gpio_select_output(GPIO *gpio, uint8_t pin, int level)
{
	if (gpio->fd < 0 || pin > 31)
		return -1;

	GPIO_ADDR reg = (GPIO_ADDR)(gpio->map) + gpio_gpfsel(pin);
	
	uint32_t val = ~(7 << ((pin % 10) * 3));
	*reg &= val; /* 000 = Input */

	val = 1 << ((pin % 10) * 3);
	*reg |= val; /* 001 = Output */

	return gpio_set(gpio, pin, level);
}

int
gpio_init(struct k8048 *k)
{
	/* TX (VPP) */
	gpio_select_output(&k->gpio, k->gpio.tx, k->bitrules & VPP_OUT_FLIP);

	/* RTS (CLOCK) */
	gpio_select_output(&k->gpio, k->gpio.rts, k->bitrules & PGC_OUT_FLIP);

	/* DTR (DATA OUT) */
	gpio_select_output(&k->gpio, k->gpio.dtr, k->bitrules & PGD_OUT_FLIP);

	/* CTS (DATA IN) */
	gpio_select_input(&k->gpio, k->gpio.cts, GPPUD_UP);

	return 0;
}

void
gpio_test(GPIO *gpio, int seconds)
{
	uint8_t output_level = 0, input_level;

	printf("\nTEST MODE 4 [RPI GPIO] CTRL-C TO STOP\n");

	while (1) {
		printf("\n");

		gpio_set(gpio, gpio->tx,  output_level);
		printf("GPIO %d (TX)  = %d\n", gpio->tx, output_level);

		gpio_set(gpio, gpio->rts, output_level);
		printf("GPIO %d (RTS) = %d\n", gpio->rts, output_level);

		gpio_set(gpio, gpio->dtr, output_level);
		printf("GPIO %d (DTR) = %d\n", gpio->dtr, output_level);

		gpio_get(gpio, gpio->cts, &input_level);
		printf("GPIO %d (CTS) = %d\n", gpio->cts, input_level);

		fflush(stdout);
		
		sleep(seconds);
		output_level = 1 - output_level;
	}
}
